<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Preferences.php 4780 2006-05-03 15:37:53Z matteo@beccati.com $
*/

require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/www/admin/lib-permissions.inc.php';

/**
 * A preferences management class for the Max administration interface.
 *
 * @package    Max
 * @author     Andrew Hill <andrew@m3.net>
 * @TODO Class needs to be updated so that it will load the admin
 * preferences for items that are not set in the agency preferences.
 */
class MAX_Admin_Preferences
{
    var $prefSql;
    
    /**
     * The constructor method. Creates an array so that updates to
     * agency preferences can be (locally) stored.
     */
    function MAX_Admin_Preferences()
    {
        $this->prefSql = array();
    }

    /**
     * A method to load the preference data stored in the database
     * into a global array ($GLOBALS['_MAX']['PREF']).
     *
     * @static
     * @param integer $agencyId Optional agency ID.
     * @return array Returns an array containing the preferences loaded
     *               into the global array.
     */
    function loadPrefs($agencyId = null)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if (is_null($agencyId)) {
            if (phpAds_isUser(phpAds_Agency)) {
                $agencyId = phpAds_getUserID();
            } else {
                $agencyId = 0;
            }
        }
        $dbh =& MAX_DB::singleton();
        $query = "
            SELECT
                *
            FROM
                {$conf['table']['prefix']}{$conf['table']['preference']}
            WHERE
                agencyid = $agencyId
        ";
        $result = $dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE);
        }
        if ($result->numRows() == 1) {
            $row = $result->fetchRow();
            foreach ($row as $key => $value) {
                $GLOBALS['_MAX']['PREF'][$key] = $value;
            }

            if (phpAds_isUser(phpAds_Client)) {
                MAX_Admin_Preferences::loadEntityPrefs('advertiser');
            } elseif (phpAds_isUser(phpAds_Affiliate)) {
                MAX_Admin_Preferences::loadEntityPrefs('publisher');
            }
            
            return $GLOBALS['_MAX']['PREF'];
        }
        return array();
    }
    
    /**
     * A method to load entity preference data stored in the database
     * as key/value pairs and merge it into a global array
     * ($GLOBALS['_MAX']['PREF'])
     *
     * @static
     * @param string Entity type (currently supported: advertiser, publisher)
     * @param int The entity unique id, defaults to current user id
     */
    function loadEntityPrefs($entityType, $entityId = '')
    {
        $conf = $GLOBALS['_MAX']['CONF'];

        if (empty($entityId)) {
            $entityId = phpAds_getUserId();
        }

        switch ($entityType) {
        
        case 'advertiser':    
            $table_name   = "{$conf['table']['prefix']}{$conf['table']['preference_advertiser']}";
            $table_column = 'advertiser_id';
            break;
        
        case 'publisher':    
            $table_name   = "{$conf['table']['prefix']}{$conf['table']['preference_publisher']}";
            $table_column = 'publisher_id';
            break;
        
        default:
            MAX::raiseError("The MAX_Admin_Preferences module discovered an entity type that it didn't know how to handle.", MAX_ERROR_INVALIDARGS);
        }
        
        $prefs = & new MAX_Admin_Preferences_Variables;
        
        $dbh =& MAX_DB::singleton();
        $query = "
            SELECT
                preference,
                value
            FROM
                {$table_name}
            WHERE
                {$table_column} = '$entityId'
        ";
        $result = $dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE);
        }
        
        while ($row = $result->fetchRow()) {
            if ($prefs->exists($row['preference'])) {
                $GLOBALS['_MAX']['PREF'][$row['preference']] = $prefs->cast($row['value'], $row['preference']);
            }
        }
    }
    /**
     * A method to write entity preference data  in the database
     * as key/value pairs
     *
     * @static
     * @param string Entity type (currently supported: advertiser, publisher)
     * @param int The entity unique id, defaults to current user id
     * @return boolean True when the preferences were correctly saved, false otherwise.
     */
    function writeEntityPrefs($entityType, $entityId = '')
    {
        if (!is_null($this->prefSql)) {
            $conf = $GLOBALS['_MAX']['CONF'];
            $dbh =& MAX_DB::singleton();
    
            if (empty($entityId)) {
                $entityId = phpAds_getUserId();
            }
    
            switch ($entityType) {
            
            case 'advertiser':    
                $table_name   = "{$conf['table']['prefix']}{$conf['table']['preference_advertiser']}";
                $table_column = 'advertiser_id';
                break;
            
            case 'publisher':    
                $table_name   = "{$conf['table']['prefix']}{$conf['table']['preference_publisher']}";
                $table_column = 'publisher_id';
                break;
            
            default:
                MAX::raiseError("The MAX_Admin_Preferences module discovered an entity type that it didn't know how to handle.", MAX_ERROR_INVALIDARGS);
            }
            
            // Try to INSERT first
            foreach ($this->prefSql as $key => $value) {
                $query = "
                    INSERT INTO {$table_name} (
                        {$table_column},
                        preference,
                        value
                    ) VALUES (
                        '$entityId',
                        '$key',
                        '$value'
                    )
                    ";
                // Don't use a PEAR_Error handler
                PEAR::pushErrorHandling(null);
                $result = $dbh->query($query);
                // Resore the PEAR_Error handler
                PEAR::popErrorHandling();
                if (PEAR::isError($result)) {
                    // Can't INSERT, try UPDATE instead
                    $query = "
                        UPDATE
                            {$table_name}
                        SET
                            value = '$value'
                        WHERE
                            {$table_column} = '{$entityId}' AND
                            preference = '{$key}'
                        ";
                    $result = $dbh->query($query);
                    if (PEAR::isError($result)) {
                        return MAX::raiseError($result, MAX_ERROR_DBFAILURE);
                    }
                }
            }
            unset($this->prefSql);
        }
        
        return true;
    }
        
    /**
     * A method for defining required changes to the agency preferences.
     *
     *
     * @param string $pref The preference value to change.
     * @param string $value The new value for the preference.
     */
    function setPrefChange($pref, $value)
    {
        if (phpAds_isUser(phpAds_Agency)) {
            $agencyid = phpAds_getUserID();
        } else {
            $agencyid = 0;
        }
        $this->prefSql[$pref] = $value;
    }
    
    /**
     * A method for writing out any required changes to an agency's preferences.
     *
     * @return boolean True when the preferences were correctly saved,
     *                 false otherwise.
     */
    function writePrefChange()
    {
        if (!is_null($this->prefSql)) {
            $conf = $GLOBALS['_MAX']['CONF'];
            $dbh =& MAX_DB::singleton();

            if (phpAds_isUser(phpAds_Client)) {
                return MAX_Admin_Preferences::writeEntityPrefs('advertiser');
            } elseif (phpAds_isUser(phpAds_Affiliate)) {
                return MAX_Admin_Preferences::writeEntityPrefs('publisher');
            } else {
                $agencyId   = phpAds_isUser(phpAds_Agency) ? phpAds_getUserID() : 0;
            }
                        
            // Try to INSERT first
            $query = "
                INSERT INTO
                    {$conf['table']['prefix']}{$conf['table']['preference']}
                    (
                    agencyid,
                ";
            foreach ($this->prefSql as $key => $value) {
                $query .= "$key, ";
            }
            $query = preg_replace('/, $/', '', $query);
            $query .= "
                    )
                VALUES
                    (
                    '{$agencyId}',
                ";
            foreach ($this->prefSql as $value) {
                $query .= "'$value', ";
            }
            $query = preg_replace('/, $/', '', $query);
            $query .= '
                    )
                ';
            // Don't use a PEAR_Error handler
            PEAR::pushErrorHandling(null);
            $result = $dbh->query($query);
            // Resore the PEAR_Error handler
            PEAR::popErrorHandling();
            if (PEAR::isError($result)) {
                // Can't INSERT, try UPDATE instead
                foreach ($this->prefSql as $key => $value) {
                    $sql[] = "$key = '$value'";
                }
                $query = "
                    UPDATE
                        {$conf['table']['prefix']}{$conf['table']['preference']}
                    SET
                        " . join(', ', $sql) . "
                    WHERE
                        agencyid = '{$agencyId}'
                    ";
                $result = $dbh->query($query);
                if (PEAR::isError($result)) {
                    return MAX::raiseError($result, MAX_ERROR_DBFAILURE);
                }
            }
            unset($this->prefSql);
        }
        return true;
    }

}

/**
 * A preferences class to store preference variables type and defaults
 * for the Max administration interface.
 *
 * @package    Max
 * @author     Matteo Beccati <matteo@beccati.com>
 */
class MAX_Admin_Preferences_Variables
{
    var $_prefs = array(
        'my_header'                            => array('type' => 'string'),
        'my_footer'                            => array('type' => 'string'),
        'my_logo'                            => array('type' => 'string'),
        'language'                            => array('type' => 'string',    'default' => 'english'),
        'name'                                => array('type' => 'string'),
        'company_name'                        => array('type' => 'string',    'default' => 'mysite.com'),
        'override_gd_imageformat'            => array('type' => 'string'),
        'begin_of_week'                        => array('type' => 'int',        'default' => '1'),
        'percentage_decimals'                => array('type' => 'int',        'default' => '2'),
        'type_sql_allow'                    => array('type' => 'bool',        'default' => 't'),
        'type_url_allow'                    => array('type' => 'bool',        'default' => 't'),
        'type_web_allow'                    => array('type' => 'bool',        'default' => 'f'),
        'type_html_allow'                    => array('type' => 'bool',        'default' => 't'),
        'type_txt_allow'                    => array('type' => 'bool',        'default' => 't'),
        'banner_html_auto'                    => array('type' => 'bool',        'default' => 't'),
        'admin'                                => array('type' => 'string',    'default' => 'phpadsuser'),
        'admin_pw'                            => array('type' => 'string',    'default' => 'phpadspass'),
        'admin_fullname'                    => array('type' => 'string',    'default' => 'Your Name'),
        'admin_email'                        => array('type' => 'string',    'default' => 'your@email.com'),
        'warn_admin'                        => array('type' => 'bool',        'default' => 't'),
        'warn_agency'                        => array('type' => 'bool',        'default' => 't'),
        'warn_client'                        => array('type' => 'bool',        'default' => 't'),
        'warn_limit'                        => array('type' => 'int',        'default' => '100'),
        'admin_email_headers'                => array('type' => 'string'),
        'admin_novice'                        => array('type' => 'bool',        'default' => 't'),
        'default_banner_weight'                => array('type' => 'int',        'default' => '1'),
        'default_campaign_weight'            => array('type' => 'int',        'default' => '1'),
        'default_banner_url'                => array('type' => 'string'),
        'default_banner_destination'        => array('type' => 'string'),
        'client_welcome'                    => array('type' => 'bool',        'default' => 't'),
        'client_welcome_msg'                => array('type' => 'string'),
        'content_gzip_compression'            => array('type' => 'bool',        'default' => 'f'),
        'userlog_email'                        => array('type' => 'bool',        'default' => 't'),
        'gui_show_campaign_info'            => array('type' => 'bool',        'default' => 't'),
        'gui_show_campaign_preview'            => array('type' => 'bool',        'default' => 'f'),
        'gui_campaign_anonymous'            => array('type' => 'bool',        'default' => 'f'),
        'gui_show_banner_info'                => array('type' => 'bool',        'default' => 't'),
        'gui_show_banner_preview'            => array('type' => 'bool',        'default' => 't'),
        'gui_show_banner_html'                => array('type' => 'bool',        'default' => 'f'),
        'gui_show_matching'                    => array('type' => 'bool',        'default' => 't'),
        'gui_show_parents'                    => array('type' => 'bool',        'default' => 'f'),
        'gui_hide_inactive'                    => array('type' => 'bool',        'default' => 'f'),
        'gui_link_compact_limit'            => array('type' => 'int',        'default' => '50'),
        'gui_header_background_color'        => array('type' => 'string'),
        'gui_header_foreground_color'        => array('type' => 'string'),
        'gui_header_active_tab_color'        => array('type' => 'string'),
        'gui_header_text_color'                => array('type' => 'string'),
        'gui_invocation_3rdparty_default'    => array('type' => 'int',        'default' => '0'),
        'qmail_patch'                        => array('type' => 'bool',        'default' => 'f'),
        'updates_frequency'                    => array('type' => 'int',        'default' => '7'),
        'updates_timestamp'                    => array('type' => 'int',        'default' => '0'),
        'updates_last_seen'                    => array('type' => 'string'),
        'allow_invocation_plain'            => array('type' => 'bool',        'default' => 'f'),
        'allow_invocation_plain_nocookies'     => array('type' => 'bool',        'default' => 't'),
        'allow_invocation_js'                => array('type' => 'bool',        'default' => 't'),
        'allow_invocation_frame'            => array('type' => 'bool',        'default' => 'f'),
        'allow_invocation_xmlrpc'            => array('type' => 'bool',        'default' => 'f'),
        'allow_invocation_local'            => array('type' => 'bool',        'default' => 't'),
        'allow_invocation_interstitial'     => array('type' => 'bool',        'default' => 't'),
        'allow_invocation_popup'            => array('type' => 'bool',        'default' => 't'),
        'allow_invocation_clickonly'         => array('type' => 'bool',        'default' => 't'),
        'auto_clean_tables'                    => array('type' => 'bool',        'default' => 'f'),
        'auto_clean_tables_interval'         => array('type' => 'int',        'default' => '5'),
        'auto_clean_userlog'                => array('type' => 'bool',        'default' => 'f'),
        'auto_clean_userlog_interval'         => array('type' => 'int',        'default' => '5'),
        'auto_clean_tables_vacuum'            => array('type' => 'bool',        'default' => 't'),
        'autotarget_factor'                    => array('type' => 'float',        'default' => '-1'),
        'maintenance_timestamp'                => array('type' => 'int',        'default' => '0'),
        'compact_stats'                        => array('type' => 'bool',        'default' => 't'),
        'statslastday'                        => array('type' => 'string',    'default' => '1970-01-01'),
        'statslasthour'                        => array('type' => 'int',        'default' => '0'),
        'default_tracker_status'            => array('type' => 'int',        'default' => '1'),
        'gui_show_requests'                    => array('type' => 'int',        'default' => '15'),
        'gui_show_impressions'                => array('type' => 'int',        'default' => '15'),
        'gui_show_clicks'                    => array('type' => 'int',        'default' => '15'),
        'gui_show_ctr'                        => array('type' => 'int',        'default' => '15'),
        'gui_show_conversions'                => array('type' => 'int',        'default' => '15'),
        'gui_show_sr'                        => array('type' => 'int',        'default' => '15')
    );

    /**
     * Returns the variable type
     *
     * @param string Preference variable name
     * @return mixed The variable type as string or false on error
     */
    function getType($variableName)
    {
        if (isset($this->_prefs[$variableName]['type'])) {
            return $this->_prefs[$variableName]['type'];
        }
        
        return false;
    }
    
    /**
     * Returns the variable default value
     *
     * @param string Preference variable name
     * @return mixed The defalut value as string or false on error
     */
    function getDefault($variableName)
    {
        if (isset($this->_prefs[$variableName]['default'])) {
            return $this->_prefs[$variableName]['default'];
        }
        
        return null;
    }
    
    /**
     * Returns the variable existence
     *
     * @param string Preference variable name
     * @return bool True if the variable exists
     */
    function exists($variableName)
    {
        return isset($this->_prefs[$variableName]);
    }
    
    /**
     * Cast a value to the correct type for the variable
     *
     * @param mixed Value to convert
     * @param string Preference variable name
     * @return mixed Converted value or null on error
     */
    function cast($value, $variableName)
    {
        switch ($this->getType($variableName)) {
            case 'int'    :    return (int)$value;
            case 'string' :    return (string)$value;
            case 'bool'   :    return $value == 't' ? 't' : 'f';
            case 'float'  :    return (float)$value;
        }
        
        return null;
    }
}

?>
