<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: ScopeSelectionField.php 4568 2006-04-04 12:40:13Z roh@m3.net $
*/

require_once MAX_PATH . '/lib/max/Admin/UI/Field.php';
require_once MAX_PATH . '/lib/max/Admin/UI/OrganisationScope.php';
require_once MAX_PATH . '/lib/max/Admin_DA.php';
require_once MAX_PATH . '/lib/max/other/common.php';

class Admin_UI_OrganisationSelectionField extends Admin_UI_Field
{
    /**
     * PHP5-style constructor
     *
     * @param array $aFieldSelectionNames A list of the predefined 'friendly' selections.
     * @param string $fieldSelectionDefault The default selection.
     */
    function __construct($name = 'OrganisationSelectionField', $defaultAdvertiser = 'all', $defaultPublisher = 'all')
    {
        $this->_name = $name;
        $oScope = new Admin_UI_OrganisationScope();
        $oScope->setAdvertiserId($defaultAdvertiser);
        $oScope->setPublisherId($defaultPublisher);
        $this->_value = $oScope;
    }
    /**
     * PHP4-style constructor
     *
     * @param array $aFieldSelectionNames A list of the predefined 'friendly' selections.
     * @param string $fieldSelectionDefault The default selection.
     */
    function Admin_UI_OrganisationSelectionField($name = 'OrganisationSelectionField', $defaultAdvertiser = 'all', $defaultPublisher = 'all')
    {
        $this->__construct($name, $defaultAdvertiser, $defaultPublisher);
    }
    function _getLinkedAdvertisers($aParams)
    {
        $aAdvertisers = array();
        $aPlacementZones = Admin_DA::getPlacementZones($aParams, false, 'placement_id');
        if (!empty($aPlacementZones)) {
            $aSubParams = array('placement_id' => implode(',', array_keys($aPlacementZones)));
            $aAdvertisers += Admin_DA::getAdvertisers($aSubParams);
        }
        $aAdZones = Admin_DA::getAdZones($aParams, false, 'ad_id');
        if (!empty($aAdZones)) {
            $aSubParams = array('ad_id' => implode(',', array_keys($aAdZones)));
            $aAdvertisers += Admin_DA::getAdvertisers($aSubParams);
        }
        
        return $aAdvertisers;
    }
    function _getLinkedPublishers($aParams)
    {
        $aPublishers = array();
        $aPlacementZones = Admin_DA::getPlacementZones($aParams, false, 'zone_id');
        if (!empty($aPlacementZones)) {
            $aSubParams = array('zone_id' => implode(',', array_keys($aPlacementZones)));
            $aPublishers += Admin_DA::getPublishers($aSubParams);
        }
        $aAdZones = Admin_DA::getAdZones($aParams, false, 'zone_id');
        if (!empty($aAdZones)) {
            $aSubParams = array('zone_id' => implode(',', array_keys($aAdZones)));
            $aPublishers += Admin_DA::getPublishers($aSubParams);
        }
        
        return $aPublishers;
    }
    function _hasAnonymousCampaigns($aParams)
    {
        $hasAnonymousCampaigns = false;
        $aParams['placement_anonymous'] = 't';
        $aPlacementZones = Admin_DA::getPlacementZones($aParams);
        if (!empty($aPlacementZones)) {
            $hasAnonymousCampaigns = true;
        } else {
            $aAdZones = Admin_DA::getAdZones($aParams);
            if (!empty($aAdZones)) {
                $hasAnonymousCampaigns = true;
            }
        }
        return $hasAnonymousCampaigns;
    }
    
    function display()
    {
        $name = $this->_name;
        $oScope = $this->_value;

        $hasAnonymousCampaigns = false;
        
        if (phpAds_isUser(phpAds_Admin)) {
            $aAdvertisers = Admin_DA::getAdvertisers();
        } elseif (phpAds_isUser(phpAds_Agency)) {
            $aParams = array('agency_id' => phpAds_getUserID());
            $aAdvertisers = Admin_DA::getAdvertisers($aParams);
        } elseif (phpAds_isUser(phpAds_Publisher)) {
            $aParams = array('publisher_id' => phpAds_getUserID(), 'placement_anonymous' => 'f');
            $hasAnonymousCampaigns = $this->_hasAnonymousCampaigns($aParams);
            $aAdvertisers = $this->_getLinkedAdvertisers($aParams);
        }
        
        if (phpAds_isUser(phpAds_Advertiser)) {
            $advertiserId = phpAds_getUserID();
            echo "
        <input type='hidden' name='{$name}_advertiser' id='{$name}_advertiser' value='$advertiserId'>";
        } else {
            echo "
        <select name='{$name}_advertiser' id='{$name}_advertiser' tabindex='" . $this->_tabIndex++ . "'>
            <option value='all'>-- All Advertisers --</option>";
            if ($hasAnonymousCampaigns) {
                echo "
            <option value='anon'>-- Anonymous Advertisers --</option>";
            }
            foreach ($aAdvertisers as $advertiserId=>$aAdvertiser) {
                echo "
            <option value='$advertiserId'>{$aAdvertiser['name']}</option>";
            }
            echo "
        </select>";
        }
        
        if (!phpAds_isUser(phpAds_Advertiser) && !phpAds_isUser(phpAds_Publisher)) {
            echo "
        <br /><br />";
        }

        
        if (phpAds_isUser(phpAds_Admin)) {
            $aPublishers = Admin_DA::getAdvertisers();
        } elseif (phpAds_isUser(phpAds_Agency)) {
            $aParams = array('agency_id' => phpAds_getUserID());
            $aPublishers = Admin_DA::getPublishers($aParams);
        } elseif (phpAds_isUser(phpAds_Advertiser)) {
            $aParams = array('advertiser_id' => phpAds_getUserID(), 'placement_anonymous' => 'f');
            $hasAnonymousCampaigns = $this->_hasAnonymousCampaigns($aParams);
            $aPublishers = $this->_getLinkedPublishers($aParams);
        }
        
        if (phpAds_isUser(phpAds_Publisher)) {
            echo "
        <input type='hidden' name='{$name}_publisher' id='{$name}_publisher' value='$publisherId'>";
        } else {
            echo "
        <select name='{$name}_publisher' id='{$name}_publisher' tabindex='" . $this->_tabIndex++ . "'>
            <option value='all'>-- All Publishers --</option>";
            if ($hasAnonymousCampaigns) {
                echo "
            <option value='anon'>-- Anonymous Publishers --</option>";
            }
            foreach ($aPublishers as $publisherId=>$aPublisher) {
                echo "
            <option value='$publisherId'>{$aPublisher['name']}</option>";
            }
            echo "
        </select>";
        }
    }
    function setValueFromArray($aFieldValues)
    {
        $name = $this->_name;
        $oScope =& $this->_value;
        
        $publisherFieldName = $name . '_publisher';
        $advertiserFieldName = $name . '_advertiser';
        
        $anonymous = false;

        // Get advertiser scope...
        if (!empty($aFieldValues[$advertiserFieldName])) {
            if (is_numeric($aFieldValues[$advertiserFieldName])) {
                $advertiserId = $aFieldValues[$advertiserFieldName];
            } else {
                $advertiserId = false;
                if ($aFieldValues[$advertiserFieldName] == 'anon') {
                    $anonymous = true;
                }
            }
            $oScope->setAdvertiserId($advertiserId);
        }
        // Get publisher scope...
        if (!empty($aFieldValues[$publisherFieldName])) {
            if (is_numeric($aFieldValues[$publisherFieldName])) {
                $publisherId = $aFieldValues[$publisherFieldName];
            } else {
                $publisherId = false;
                if ($aFieldValues[$advertiserFieldName] == 'anon') {
                    $anonymous = true;
                }
            }
            $oScope->setPublisherId($publisherId);
        }
        $oScope->setAnonymous($anonymous);
    }
}

?>
