<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: cookie.php 4508 2006-03-27 10:33:04Z chris@m3.net $
*/

/**
 * @package    MaxDelivery
 * @subpackage cookie
 * @author     Chris Nutting <chris@m3.net>
 */


/**
 * Set a cookie in the global cookie cache
 * 
 * @param string $name name of cookie to be set
 * @param string $value value to be stored in the cookie
 * @param string $expire timestamp at which the cookie should be set to expire
 * 
 */
function MAX_cookieSet($name, $value, $expire = 0)
{
    $cookieCache =& $GLOBALS['_MAX']['COOKIE']['CACHE'];
    if (!isset($cookieCache)) {
        $cookieCache = array();
    }
    $cookieCache[$name] = array($value, $expire);
}

/**
 * This function does a set cookie and redirect to self to test setting cookies
 *
 * @param string $viewerId The viewerId value to attempt to set
 */
function MAX_cookieSetViewerIdAndRedirect($viewerId) {
    $conf = $GLOBALS['_MAX']['CONF'];
    
    MAX_cookieSet($conf['var']['viewerId'], $viewerId, time()+365*24*60*60);
    MAX_cookieFlush();
    
    // Determine if the access to Max was made using HTTPS
    if ($_SERVER['SERVER_PORT'] == $conf['max']['sslPort']) {
        $url = constructSecureDeliveryUrl(basename($_SERVER['PHP_SELF']));
    } else {
        $url = constructDeliveryUrl(basename($_SERVER['PHP_SELF']));
    }
    $url .= "?{$conf['var']['cookieTest']}=1&" . $_SERVER['QUERY_STRING'];
    header("Location: {$url}");
    exit;
}

/**
 * Send all cookies in the global cookie cache to the browser
 * 
 */ 
function MAX_cookieFlush()
{
    $conf = $GLOBALS['_MAX']['CONF'];
    $cookieCache =& $GLOBALS['_MAX']['COOKIE']['CACHE'];
    
    MAX_cookieSendP3PHeaders();
    if (!empty($cookieCache)) {
        // Set cookies
        while (list($name,$v) = each ($cookieCache)) {
            list($value, $expire) = $v;
            setcookie($name, $value, $expire, '/', (!empty($conf['cookie']['domain']) ? $conf['cookie']['domain'] : null));
        }
        // Clear cache
        $cookieCache = array();
    }
    
    // Compact all individual cookies into packed except for any cookies for the current bannerid
    // We only need to set these packed cookies if new capping data has been merged
    $cookieNames = array($conf['var']['blockAd'], $conf['var']['capAd'], $conf['var']['sessionCapAd']);
    $now = time();
    // For each type of cookie, repack if necessary
    foreach ($cookieNames as $cookieName) {
        // We only need to write out the compacted cookie if a new item is to be inserted (or updated)
        if (empty($_COOKIE["_{$cookieName}"])) {
            continue;
        }
        switch ($cookieName) {
            case $conf['var']['blockAd']      : $expire = $now + 30*24*60*60; break;
            case $conf['var']['capAd']        : $expire = $now + 365*24*60*60; break;
            case $conf['var']['sessionCapAd'] : $expire = 0; break;
        }
        if (!empty($_COOKIE[$cookieName]) && is_array($_COOKIE[$cookieName])) {
            $data = array();
            foreach ($_COOKIE[$cookieName] as $adId => $value) {
                $data[] = "{$adId}.{$value}";
            }
            setcookie($cookieName, implode('_', $data), $expire, '/', (!empty($conf['cookie']['domain']) ? $conf['cookie']['domain'] : null));
        }
    }   
}

/**
 * This function unpacks the serialized array used for capping
 *
 */
function MAX_cookieUnpackCapping()
{
    $conf = $GLOBALS['_MAX']['CONF'];
    
    $cookieNames = array($conf['var']['blockAd'], $conf['var']['capAd'], $conf['var']['sessionCapAd']);
    
    // For each type of cookie, unpack and add any newly set cookies to this array
    foreach ($cookieNames as $cookieName) {
        if (!empty($_COOKIE[$cookieName])) {
            if (!is_array($_COOKIE[$cookieName])) {
                $output = array();
                $data = explode('_', $_COOKIE[$cookieName]);
                foreach ($data as $pair) {
                    list($name, $value) = explode('.', $pair);
                    $output[$name] = $value;
                }
                $_COOKIE[$cookieName] = $output;
            }
        }
        if (!empty($_COOKIE['_' . $cookieName]) && is_array($_COOKIE['_' . $cookieName])) {
            foreach ($_COOKIE['_' . $cookieName] as $adId => $cookie) {
                if ($cookieName == $conf['var']['blockAd']) {
                    $_COOKIE[$cookieName][$adId] = $cookie;
                } else {
                    $_COOKIE[$cookieName][$adId] += $cookie;
                }
                MAX_cookieSet("_{$cookieName}[{$adId}]", false);
            }
        }
    }
}

/**
 * This function gets the unique user ID (creating if necessary)
 *
 * @param bool $create Should a viewer ID be created if not present in $_COOKIE ?
 * @return string The viewer ID
 */
function MAX_cookieGetUniqueViewerID($create = true)
{
    $conf = &$GLOBALS['_MAX']['CONF'];
    if (isset($_COOKIE[$conf['var']['viewerId']])) {
        $userid = $_COOKIE[$conf['var']['viewerId']];
    } else {
        if ($create) {
            $remote_address = $_SERVER['REMOTE_ADDR'];
            $local_address  = $conf['webpath']['delivery']; // How do I get the IP address of this server?
            // Get the exact time
            list($usec, $sec) = explode(" ", microtime());
            $time = (float) $usec + (float) $sec;
            // Get a random number
            $random = mt_rand(0,999999999);
            $userid = substr(md5($local_address.$time.$remote_address.$random),0,32);  // Need to find a way to generate this...
            $GLOBALS['_MAX']['COOKIE']['newViewerId'] = true;
        } else {
            $userid = null;
        }
    }
    return $userid;
}

/**
 * This function generates a psudo-unique user ID based on IP address and user-agent
 *
 * @return string The cookieless viewer ID
 */
function MAX_cookieGetCookielessViewerID()
{
    $remote_address = $_SERVER['REMOTE_ADDR'];
    $user_agent = (!empty($_SERVER['HTTP_USER_AGENT'])) ? $_SERVER['HTTP_USER_AGENT'] : '';
    //$http_accept = (!empty($_SERVER['HTTP_ACCEPT'])) ? $_SERVER['HTTP_ACCEPT'] : '';
    $http_accept_language = (!empty($_SERVER['HTTP_ACCEPT_LANGUAGE'])) ? $_SERVER['HTTP_ACCEPT_LANGUAGE'] : '';
    $http_accept_encoding = (!empty($_SERVER['HTTP_ACCEPT_ENCODING'])) ? $_SERVER['HTTP_ACCEPT_ENCODING'] : '';
    return MAX_COOKIELESS_PREFIX . substr(md5($remote_address.$user_agent.$http_accept_language.$http_accept_encoding), 0, 32-(strlen(MAX_COOKIELESS_PREFIX)));
   
}

/**
 * Send the appropriate P3P headers to attempt to permit 3rd party cookies
 *
 */
function MAX_cookieSendP3PHeaders() {
    // Send P3P headers
    $conf = $GLOBALS['_MAX']['CONF'];
    
    if ($conf['p3p']['policies']) {
		$p3p_header = '';
		if ($conf['p3p']['policyLocation'] != '') {
			$p3p_header .= " policyref=\"".$conf['p3p']['policyLocation']."\"";
		}
        if ($conf['p3p']['policyLocation'] != '' && $conf['p3p']['compactPolicy'] != '') {
            $p3p_header .= ", ";
        }
		if ($conf['p3p']['compactPolicy'] != '') {
			$p3p_header .= " CP=\"".$conf['p3p']['compactPolicy']."\"";
		}
		if ($p3p_header != '')
		header("P3P: $p3p_header");
	}
}

/**
 * This function sets any capping cookies that are required for this impression
 *
 * @param int $adId The ID of the ad to set this capping cookie for
 * @param int $block The number of seconds to prevent this ad from showing
 * @param int $capping The total number of times a viewer can see this creative
 * @param int $session_capping The number of times per session that a viewer can see this creative
 */
function MAX_cookieSetCapping($adId, $block = 0, $capping = 0, $session_capping = 0)
{
    $conf = $GLOBALS['_MAX']['CONF'];
    $timestampNow  = time();
    
    if ($block) {
        // Expire blocking cookies in 30 days
        $expire = $timestampNow + 30*24*60*60;
        
        // Block cookies record the timestamp that this ad was last seen
        $value = $timestampNow;
        MAX_cookieSet("_{$conf['var']['blockAd']}[{$adId}]", $value, $expire);
    }
    if ($capping) {
        // Set capping cookies to be "permanent"
        $expire = $timestampNow + $conf['cookie']['permCookieSeconds'];
        
        // Capping cookies increment a view counter
        $value = MAX_cookieGetCappingValue($adId, $conf['var']['capAd']) + 1;
        MAX_cookieSet("_{$conf['var']['capAd']}[{$adId}]", $value, $expire);
    }
    if ($session_capping) {
        // Expire session capping cookies at end of session
        $expire = 0;
        
        // Capping cookies increment a view counter
        $value = MAX_cookieGetCappingValue($adId, $conf['sessionCapAd']) + 1;
        MAX_cookieSet("_{$conf['var']['sessionCapAd']}[{$adId}]", $value, $expire);
    }
}

/**
 * Get the existing value (if found) for a particular capping cookie/
 *
 * @param unknown_type $adId
 * @param unknown_type $cookieName
 */
function MAX_cookieGetCappingValue($adId, $cookieName)
{
    // Get the value of the variable from either the cookie or request
    if (isset($_COOKIE[$cookieName][$adId])) {
        $value = $_COOKIE[$cookieName][$adId];
    } elseif (isset($_REQUEST[$cookieName][$adId])) {
        $value = $_REQUEST[$cookieName][$adId];
    } else {
        $value = 0;
    }
}

/**
 * Check to see if this particular ad impression needs cookies set, and set them
 *
 * 1.  Check to see if the ad is capped, and if so, increment the value by 1
 * 2.  Check to see if the ad is blocked, and if so, update the date/time that it was viewed.
 * 3.  Repeat for session based capping and blocking
 *
 * @param array $conf The configuration array
 * @param integer $adId The ID of the ad that was viewed
 */
function updateAdImpressionCookies($conf, $adId, $capping)
{
    $time = time();
    
    _setAdImpressionCookie($conf, 'capAd', $adId, $time, true, 'cap', $capping[$conf['var']['capAd']]);
    _setAdImpressionCookie($conf, 'sessionCapAd', $adId, $time, false, 'cap', $capping[$conf['var']['sessionCapAd']]);
    _setAdImpressionCookie($conf, 'blockAd', $adId, $time, true, 'block', $capping[$conf['var']['blockAd']]);
    _setAdImpressionCookie($conf, 'capAd', -1, $time, true, 'cap', 1);
}

/**
 * Check to see if a value is present in the cookie or request for a given parameter.  If it is, then update it.
 *
 * @param array $conf The configuration array
 * @param string $varName The name of the variable to check
 * @param integer $adId The ID of the ad that was viewed
 * @param integer $time The server time in UNIX format
 * @param boolean $permanent True if there should be a permanent cookie, false if there should be a session cookie
 * @param string $type If it is a capping cookie, then the value will be 'cap'.  If it is a time limitation cookie, then the value will be 'block'
 */
function _setAdImpressionCookie($conf, $varName, $adId, $time, $permanent = true, $type = 'cap', $cappingValue = 0)
{
    // We only need to continue processing if a cappingValue was passed in
    if (!$cappingValue) { return; }
        
    // Get the name of the cookie or request variable
    $name = $conf['var'][$varName];
    $cookieName = $name . '[' . $adId . ']';
    // Get the cookie expiration date
    if ($permanent) {
        $expire = $time + $conf['cookie']['permCookieSeconds'];
    } elseif ($varName == 'blockAd') {
        // Set blocking cookies to expire in 30 days
        $expire = $time + 30*24*60*60;
    } else {
        $expire = 0;
    }
    

    if ($value !== false) {
        $value = $type == 'cap' ? intval($value) + 1 : $time;
        setcookie($cookieName, $value, $expire, '/', (!empty($conf['cookie']['domain']) ? $conf['cookie']['domain'] : null));
    }
}

?>
