<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: PriorityAdServerForecastZoneImpressions.mtp.test.php 4346 2006-03-06 16:43:19Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/max/Maintenance/Priority/AdServer/tasks/ForecastZoneImpressions.php';
require_once MAX_PATH . '/lib/max/Maintenance/Priority/Entities.php';

/**
 * A class for testing the Maintenance_Priority_DeliveryLimitation class.
 *
 * @package    MaxMaintenance
 * @subpackage TestSuite
 * @author     Demian Turner <demian@m3.net>
 */
class TestOfPriorityAdserverForecastZoneImpressions extends UnitTestCase
{

    /**
     * The constructor method.
     */
    function TestOfPriorityAdserverForecastZoneImpressions()
    {
        $this->UnitTestCase();
        Mock::generate('MAX_Dal_Maintenance');
    }

    function testRun()
    {
        $obj = $this->getCurrentTask();
        $obj->run();
    }

    function getCurrentTask()
    {
        // Mock the MAX_DB class used in the constructor method
        $oDal = &new MockMAX_Dal_Maintenance($this);
        $oServiceLocator = &ServiceLocator::instance();
        $oServiceLocator->register('MAX_Dal_Maintenance', $oDal);

        return new ForecastZoneImpressions();
    }

    function testGetUpdateTypeRequired()
    {
        // TEST 1: Test false when $mtceStatsLastRun date is null
        $task = $this->getCurrentTask();
        $task->_init();
        $task->mtceStatsLastRun->oUpdatedToDate = null;
        $this->assertTrue($task->getUpdateTypeRequired());
        $this->assertFalse($task->safeInsert);
        unset($task);

        // TEST 2: Test false when $mtceStatsLastRun date is null
        $task = $this->getCurrentTask();
        $task->_init();
        // Stats must be set so we don't exit
        $task->mtceStatsLastRun->oUpdatedToDate = new Date();
        $task->mtcePriorityLastRun->oUpdatedToDate = null;
        $this->assertTrue($task->getUpdateTypeRequired());
        $this->assertTrue($task->safeInsert);
        unset($task);

        // TEST 3: Test true when operation intervals are different
        $task = $this->getCurrentTask();
        $task->_init();
        // Stats must be set so we don't exit
        $task->mtceStatsLastRun->oUpdatedToDate = new Date();
        $task->mtcePriorityLastRun->oUpdatedToDate = new Date();
        $randInterval = rand(1, 60);
        $task->mtcePriorityLastRun->operationInt =
            ($this->oOperationInterval->interval != $randInterval)
            ? $randInterval
            : $randInterval + 1;
        $this->assertTrue($task->getUpdateTypeRequired());
        $this->assertTrue($task->safeInsert);
        unset($task);

        // TEST 4: Test that when the stats last run date is before the priority
        // last run date, we get a false
        $task = $this->getCurrentTask();
        $task->_init();
        // Set intervals as identical to pass test
        $task->mtcePriorityLastRun->operationInt = $task->oOperationInterval->interval;
        // Set priority date in the future
        $task->mtceStatsLastRun->oUpdatedToDate = new Date();
        $prioDate = new Date();
        $prioDate->addSeconds(SECONDS_PER_DAY);
        $task->mtcePriorityLastRun->oUpdatedToDate = $prioDate;
        $this->assertFalse($task->getUpdateTypeRequired());
        $this->assertTrue($task->safeInsert);
        unset($task);

        // TEST 5: Test that we return true when the prio date and stats dates are greater
        // than 7 days apart
        $task = $this->getCurrentTask();
        $task->_init();
        // Set default date for stats
        $task->mtceStatsLastRun->oUpdatedToDate = new Date();
        // Set intervals as identical to pass test
        $task->mtcePriorityLastRun->operationInt = $task->oOperationInterval->interval;
        // Set prio date greater than 7 days in the past
        $prioDate = new Date();
        $prioDate->subtractSeconds(SECONDS_PER_WEEK);
        $prioDate->subtractSeconds(1);
        $task->mtcePriorityLastRun->oUpdatedToDate  = $prioDate;
        $this->assertTrue($task->getUpdateTypeRequired());
        $this->assertTrue($task->safeInsert);
    }

    function testGetUpdateTypeRequiredSameOperationInterval()
    {
        // Expect return false when operation intervals are identical
        $task = $this->getCurrentTask();
        $task->_init();
        // Set intervals as identical to pass test
        $task->mtcePriorityLastRun->operationInt = $task->oOperationInterval->interval;
        // Set prioitisation last run date
        $task->mtcePriorityLastRun->oUpdatedToDate = new Date('2005-06-22 12:59:59');
        $task->mtceStatsLastRun->oUpdatedToDate = new Date('2005-06-22 11:59:59');
        $this->assertFalse($task->getUpdateTypeRequired());
    }

    function testGetUpdateTypeRequiredIntervalsSuccessfullyReturned()
    {
        $task = $this->getCurrentTask();
        $task->_init();
        // Set intervals as identical to pass test
        $task->mtcePriorityLastRun->operationInt = $task->oOperationInterval->interval;
        // Set prio date
        $task->mtcePriorityLastRun->oUpdatedToDate = new Date();
        $statsDate = new Date();
        $statsDate->addSeconds($task->oOperationInterval->interval * SECONDS_PER_MINUTE);
        $task->mtceStatsLastRun->oUpdatedToDate = $statsDate;
        $ret = $task->getUpdateTypeRequired();
        $this->assertTrue(is_array($ret));
        list($priorityOpIntId, $statsOpIntId) = $ret;
        $this->assertNotEqual($priorityOpIntId, $statsOpIntId);
    }

    function testGetOperationIntRangeByTypeAll()
    {
        $task = $this->getCurrentTask();
        $task->_init();
        $type = true;
        $range = $task->getOperationIntRangeByType($type);
        $this->assertTrue(is_array($range));
        $oOperationInterval = &OperationInterval::singleton();
        $id = $oOperationInterval->getIntervalID($task->oDateNow);
        if ($id === 0) {
            $this->assertTrue(count($range) == 1);
            $myRange = $range[0];
            $this->assertTrue(is_array($myRange));
            $this->assertTrue(count($myRange) == 168);
        } else {
            $this->assertTrue(count($range) == 2);
            $sum = 0;
            $myRange = $range[0];
            $this->assertTrue(is_array($myRange));
            $sum += count($myRange);
            $myRange = $range[1];
            $this->assertTrue(is_array($myRange));
            $sum += count($myRange);
            $this->assertTrue($sum == 168);
        }
    }

    function testGetOperationIntRangeByTypeContiguous()
    {
        $task = $this->getCurrentTask();
        $task->_init();
        $type = array(23, 56);
        $range = $task->getOperationIntRangeByType($type);
        $this->assertTrue(is_array($range));
        $this->assertTrue(count($range) == 1);
        $myRange = $range[0];
        $this->assertTrue(is_array($myRange));
        $this->assertTrue(count($myRange) == 56 - 23);
    }

    function testGetOperationIntRangeByTypeNonContiguous()
    {
        $conf = &$GLOBALS['_MAX']['CONF'];
        $conf['maintenance']['operationInterval'] = 60 * 24;
        $start = 5;
        $end   = 1;
        $task = $this->getCurrentTask();
        $task->_init();
        $type = array($start, $end);
        $range = $task->getOperationIntRangeByType($type);
        $this->assertTrue(is_array($range));
        $this->assertTrue(count($range) == 2);
        $myRange1 = $range[0];
        $myRange2 = $range[1];
        $this->assertTrue(is_array($myRange1));
        $this->assertTrue(is_array($myRange2));
        $count1 = count($myRange1) + count($myRange2);
        $count2 = ((($task->oOperationInterval->intervalsPerWeek() - 1) - ($start)) + ($end + 1));
        $this->assertTrue($count1 == $count2);
        TestEnv::restoreConfig();
    }

    function testBuildRange()
    {
        $opInt = OperationInterval::singleton();
        $startId = 34;
        $totalIntervals = 168;
        $range = array();
        $oStartDate = new Date();
        for ($x = $startId, $y = 0; $y < $totalIntervals; $x++, $y++) {
            if ($x == $totalIntervals) {
               $x = 0;
            }
            $dates = array();
            $dates['start'] = $oStartDate->format('%Y-%m-%d %H:%M:%S');
            $oEndDate = new Date($oStartDate);
            $oEndDate->addSeconds($opInt->secondsPerInterval());
            $oEndDate->subtractSeconds(1);
            $dates['end'] = $oEndDate->format('%Y-%m-%d %H:%M:%S');
            unset($oEndDate);
            $range[$x] = $dates;
            $oStartDate->addSeconds($opInt->secondsPerInterval());
        }

        $this->assertTrue(count($range) == 168);
        $this->assertTrue(array_key_exists(0, $range));
        $this->assertTrue(array_key_exists(167, $range));

    }

    function testSliceRange()
    {
        $data = array (
          34 => '2005-05-12 14:31:42',
          35 => '2005-05-12 15:31:42',
          36 => '2005-05-12 16:31:42',
          37 => '2005-05-12 17:31:42',
          38 => '2005-05-12 18:31:42',
          39 => '2005-05-12 19:31:42',
          40 => '2005-05-12 20:31:42',
          41 => '2005-05-12 21:31:42',
          42 => '2005-05-12 22:31:42',
          43 => '2005-05-12 23:31:42',
          44 => '2005-05-13 00:31:42',
          45 => '2005-05-13 01:31:42',
          46 => '2005-05-13 02:31:42',
          47 => '2005-05-13 03:31:42',
          48 => '2005-05-13 04:31:42',
          49 => '2005-05-13 05:31:42',
          50 => '2005-05-13 06:31:42',
          51 => '2005-05-13 07:31:42',
          52 => '2005-05-13 08:31:42',
          53 => '2005-05-13 09:31:42',
          54 => '2005-05-13 10:31:42',
          55 => '2005-05-13 11:31:42',
          56 => '2005-05-13 12:31:42',
          57 => '2005-05-13 13:31:42',
          58 => '2005-05-13 14:31:42',
          59 => '2005-05-13 15:31:42',
          60 => '2005-05-13 16:31:42',
          61 => '2005-05-13 17:31:42',
          62 => '2005-05-13 18:31:42',
          63 => '2005-05-13 19:31:42',
          64 => '2005-05-13 20:31:42',
          65 => '2005-05-13 21:31:42',
          66 => '2005-05-13 22:31:42',
          67 => '2005-05-13 23:31:42',
          68 => '2005-05-14 00:31:42',
          69 => '2005-05-14 01:31:42',
          70 => '2005-05-14 02:31:42',
          71 => '2005-05-14 03:31:42',
          72 => '2005-05-14 04:31:42',
          73 => '2005-05-14 05:31:42',
          74 => '2005-05-14 06:31:42',
          75 => '2005-05-14 07:31:42',
          76 => '2005-05-14 08:31:42',
          77 => '2005-05-14 09:31:42',
          78 => '2005-05-14 10:31:42',
          79 => '2005-05-14 11:31:42',
          80 => '2005-05-14 12:31:42',
          81 => '2005-05-14 13:31:42',
          82 => '2005-05-14 14:31:42',
          83 => '2005-05-14 15:31:42',
          84 => '2005-05-14 16:31:42',
          85 => '2005-05-14 17:31:42',
          86 => '2005-05-14 18:31:42',
          87 => '2005-05-14 19:31:42',
          88 => '2005-05-14 20:31:42',
          89 => '2005-05-14 21:31:42',
          90 => '2005-05-14 22:31:42',
          91 => '2005-05-14 23:31:42',
          92 => '2005-05-15 00:31:42',
          93 => '2005-05-15 01:31:42',
          94 => '2005-05-15 02:31:42',
          95 => '2005-05-15 03:31:42',
          96 => '2005-05-15 04:31:42',
          97 => '2005-05-15 05:31:42',
          98 => '2005-05-15 06:31:42',
          99 => '2005-05-15 07:31:42',
          100 => '2005-05-15 08:31:42',
          101 => '2005-05-15 09:31:42',
          102 => '2005-05-15 10:31:42',
          103 => '2005-05-15 11:31:42',
          104 => '2005-05-15 12:31:42',
          105 => '2005-05-15 13:31:42',
          106 => '2005-05-15 14:31:42',
          107 => '2005-05-15 15:31:42',
          108 => '2005-05-15 16:31:42',
          109 => '2005-05-15 17:31:42',
          110 => '2005-05-15 18:31:42',
          111 => '2005-05-15 19:31:42',
          112 => '2005-05-15 20:31:42',
          113 => '2005-05-15 21:31:42',
          114 => '2005-05-15 22:31:42',
          115 => '2005-05-15 23:31:42',
          116 => '2005-05-16 00:31:42',
          117 => '2005-05-16 01:31:42',
          118 => '2005-05-16 02:31:42',
          119 => '2005-05-16 03:31:42',
          120 => '2005-05-16 04:31:42',
          121 => '2005-05-16 05:31:42',
          122 => '2005-05-16 06:31:42',
          123 => '2005-05-16 07:31:42',
          124 => '2005-05-16 08:31:42',
          125 => '2005-05-16 09:31:42',
          126 => '2005-05-16 10:31:42',
          127 => '2005-05-16 11:31:42',
          128 => '2005-05-16 12:31:42',
          129 => '2005-05-16 13:31:42',
          130 => '2005-05-16 14:31:42',
          131 => '2005-05-16 15:31:42',
          132 => '2005-05-16 16:31:42',
          133 => '2005-05-16 17:31:42',
          134 => '2005-05-16 18:31:42',
          135 => '2005-05-16 19:31:42',
          136 => '2005-05-16 20:31:42',
          137 => '2005-05-16 21:31:42',
          138 => '2005-05-16 22:31:42',
          139 => '2005-05-16 23:31:42',
          140 => '2005-05-17 00:31:42',
          141 => '2005-05-17 01:31:42',
          142 => '2005-05-17 02:31:42',
          143 => '2005-05-17 03:31:42',
          144 => '2005-05-17 04:31:42',
          145 => '2005-05-17 05:31:42',
          146 => '2005-05-17 06:31:42',
          147 => '2005-05-17 07:31:42',
          148 => '2005-05-17 08:31:42',
          149 => '2005-05-17 09:31:42',
          150 => '2005-05-17 10:31:42',
          151 => '2005-05-17 11:31:42',
          152 => '2005-05-17 12:31:42',
          153 => '2005-05-17 13:31:42',
          154 => '2005-05-17 14:31:42',
          155 => '2005-05-17 15:31:42',
          156 => '2005-05-17 16:31:42',
          157 => '2005-05-17 17:31:42',
          158 => '2005-05-17 18:31:42',
          159 => '2005-05-17 19:31:42',
          160 => '2005-05-17 20:31:42',
          161 => '2005-05-17 21:31:42',
          162 => '2005-05-17 22:31:42',
          163 => '2005-05-17 23:31:42',
          164 => '2005-05-18 00:31:42',
          165 => '2005-05-18 01:31:42',
          166 => '2005-05-18 02:31:42',
          167 => '2005-05-18 03:31:42',
          0 => '2005-05-18 04:31:42',
          1 => '2005-05-18 05:31:42',
          2 => '2005-05-18 06:31:42',
          3 => '2005-05-18 07:31:42',
          4 => '2005-05-18 08:31:42',
          5 => '2005-05-18 09:31:42',
          6 => '2005-05-18 10:31:42',
          7 => '2005-05-18 11:31:42',
          8 => '2005-05-18 12:31:42',
          9 => '2005-05-18 13:31:42',
          10 => '2005-05-18 14:31:42',
          11 => '2005-05-18 15:31:42',
          12 => '2005-05-18 16:31:42',
          13 => '2005-05-18 17:31:42',
          14 => '2005-05-18 18:31:42',
          15 => '2005-05-18 19:31:42',
          16 => '2005-05-18 20:31:42',
          17 => '2005-05-18 21:31:42',
          18 => '2005-05-18 22:31:42',
          19 => '2005-05-18 23:31:42',
          20 => '2005-05-19 00:31:42',
          21 => '2005-05-19 01:31:42',
          22 => '2005-05-19 02:31:42',
          23 => '2005-05-19 03:31:42',
          24 => '2005-05-19 04:31:42',
          25 => '2005-05-19 05:31:42',
          26 => '2005-05-19 06:31:42',
          27 => '2005-05-19 07:31:42',
          28 => '2005-05-19 08:31:42',
          29 => '2005-05-19 09:31:42',
          30 => '2005-05-19 10:31:42',
          31 => '2005-05-19 11:31:42',
          32 => '2005-05-19 12:31:42',
          33 => '2005-05-19 13:31:42',
        );

        $highId = 67;
        $lowId = 23;
        $limit = 167;
        $range1 = array();
        for ($x = $highId; $x <= $limit; $x++) {
            $range1[$x] = $data[$x];
        }
        $range2 = array();
        for ($x = 0; $x <= $lowId; $x++) {
            $range2[$x] = $data[$x];
        }
        $this->assertTrue(is_array($range1));
        $this->assertTrue(count($range1));

        $this->assertTrue(is_array($range2));
        $this->assertTrue(count($range2));
    }

    function testInit()
    {
        $task = $this->getCurrentTask();
        $task->_init();
        $this->assertTrue(is_array($task->conf));
        $this->assertTrue(count($task->conf));

        $this->assertTrue(is_object($task->oDateNow));
        $this->assertTrue(is_a($task->oDateNow, 'date'));

        $this->assertTrue(is_object($task->mtceStatsLastRun));
        $this->assertTrue(is_a($task->mtceStatsLastRun, 'MtceStatsLastRun'));

        $this->assertTrue(is_object($task->mtcePriorityLastRun));
        $this->assertTrue(is_a($task->mtcePriorityLastRun, 'MtcePriorityLastRun'));
    }

    function testWeekSpan()
    {
        $span = new Date_Span();
        $start = new Date(date("Y-m-d H:i:s"));
        $end = new Date('2005-05-01 16:35:14');
        $oStartDateCopy = new Date();
        $oStartDateCopy->copy($start);
        $oEndDateCopy = new Date();
        $oEndDateCopy->copy($end);
        $span->setFromDateDiff($oStartDateCopy, $oEndDateCopy);
        $this->assertTrue($span->day > 7);
    }

    /**
     * Test 1 - make sure correct Zone objects are being created
     */
    function testGetActiveZones()
    {
        // mock DAL and operation interval creator methods
        Mock::generatePartial('ForecastZoneImpressions', $mockName = 'ForecastZoneImpressions' . rand(),
            array('_getDal'));
        $task = &new $mockName($this);

        // mock DAL return value
        Mock::generatePartial('MAX_Dal_Maintenance', $dalMock = 'PartialMAX_Dal_Maintenance' . rand(),
            array('getActiveZones'));
        $mockDAL = & new $dalMock($this);
        $aTestZones = array(
            array('zoneid' => 1),
            array('zoneid' => 2),
            array('zoneid' => 3),
            array('zoneid' => 4),
        );

        // set return value of DAL
        $mockDAL->setReturnValue('getActiveZones', $aTestZones);
        $task->setReturnValue('_getDal', $mockDAL);

        // run main  mock constructor
        $task->ForecastZoneImpressions();

        // run test method
        $aTestResult = $task->getActiveZones();

        // test results
        foreach ($aTestResult as $key => $oZone) {
            $this->assertIsA($oZone, 'Zone');
            $this->assertEqual($oZone->id, ($key + 1));
        }
    }

    /**
     * Test 1 - test correct date offset is being applied
     */
    function testGetTrendEndDate()
    {
        // Test 1
        // set mock return value
        $secondsPerInterval = (60 * 60);

        // create operation interval Mock
        Mock::generatePartial('OperationInterval', 'PartialMockOperationInterval',
            array('secondsPerInterval'));
        $mockOperationInterval = & new PartialMockOperationInterval($this);
        // set return value
        $mockOperationInterval->setReturnValue('secondsPerInterval', $secondsPerInterval);

        // mock DAL and operation interval creator methods
        Mock::generatePartial('ForecastZoneImpressions', 'MockForecastZoneImpressions',
            array('_getOperationIntUtils', '_getDal'));

        // create mock of main class
        $task = &new MockForecastZoneImpressions($this);
        // set return value of OperationInterval get function to be mock
        $task->setReturnValue('_getOperationIntUtils', $mockOperationInterval);
        // suppress DB errors by overiding connection functionality
        $task->setReturnValue('_getDal', true);
        // call constructor of main mock class
        $task->ForecastZoneImpressions();

        // calculate what function should return
        $oExpectedResult = new Date();
        $oExpectedResult->subtractSeconds(ZONE_FORECAST_TREND_OFFSET * $secondsPerInterval);

        // create test date
        $oEndDate = new Date();

        // run method test
        $oResultTestDate = $task->getTrendEndDate($oEndDate);

        // test values are the same
        $this->assertEqual($oResultTestDate, $oExpectedResult);
    }

    /**
     * Test 1 - test correct date offset is being applied
     */
    function testGetTrendStartDate()
    {
        // Test 1
        // set mock return value
        $secondsPerInterval = (60 * 60);

        // create operation interval Mock
        Mock::generatePartial('OperationInterval', 'PartialMockOperationInterval2',
            array('secondsPerInterval'));
        $mockOperationInterval = & new PartialMockOperationInterval2($this);
        // set return value
        $mockOperationInterval->setReturnValue('secondsPerInterval', $secondsPerInterval);

        // mock DAL and operation interval creator methods
        Mock::generatePartial('ForecastZoneImpressions', 'MockForecastZoneImpressions2',
            array('_getOperationIntUtils', '_getDal'));

        // create mock of main class
        $task = &new MockForecastZoneImpressions2($this);
        // set return value of OperationInterval get function to be mock
        $task->setReturnValue('_getOperationIntUtils', $mockOperationInterval);
        // suppress DB errors by overiding connection functionality
        $task->setReturnValue('_getDal', true);
        // call constructor of main mock class
        $task->ForecastZoneImpressions();

        // calculate what function should return
        $oExpectedResult = new Date();
        $oExpectedResult->subtractSeconds((ZONE_FORECAST_TREND_OFFSET + ZONE_FORECAST_TREND_OPERATION_INTERVALS - 1)
            * $secondsPerInterval);

        // create test date
        $oEndDate = new Date();

        // run method test
        $oResultTestDate = $task->getTrendStartDate($oEndDate);

        // test values are the same
        $this->assertEqual($oResultTestDate, $oExpectedResult);
    }

    function testCalculateForecast()
    {
        $task = &new ForecastZoneImpressions();
        $task->_init();

        // Use real zone objects to test what will really happen

        // Test 1 - all values exist
        $oZone = new Zone();
        $oZone->setIntervalIdImpressionAverage(0, 300);
        $oZone->setIntervalIdImpressionForecast(0, 400);
        $oZone->setIntervalIdImpressionActual(0, 200);
        $this->assertEqual($task->calculateForecast($oZone, 0), 150);
        unset($oZone);

        // Test 2 - test rounding
        $oZone = new Zone();
        $oZone->setIntervalIdImpressionAverage(0, 582);
        $oZone->setIntervalIdImpressionForecast(0, 4685);
        $oZone->setIntervalIdImpressionActual(0, 2545);
        $this->assertEqual($task->calculateForecast($oZone, 0), 316);
        unset($oZone);

        // Test 3 - just average impressions set
        $oZone = new Zone();
        $oZone->setIntervalIdImpressionAverage(0, 300);
        $this->assertEqual($task->calculateForecast($oZone, 0), 300);
        unset($oZone);

        // Test 4 - no average impressions - all other set
        $oZone = new Zone();
        $oZone->setIntervalIdImpressionForecast(0, 400);
        $oZone->setIntervalIdImpressionActual(0, 200);
        $this->assertEqual($task->calculateForecast($oZone, 0), 10);
        unset($oZone);

        // Test 5 - nothing set
        $oZone = new Zone();
        $this->assertEqual($task->calculateForecast($oZone, 0), 10);
        unset($oZone);

    }

    function testSetZonesImpressionAverage()
    {
        // use real zone object as could not get mock to work
        $aZone[] = new Zone(array('zoneid' => 1));
        $aZone[] = new Zone(array('zoneid' => 2));

        // use real PEAR::Date objects for start & end
        $oStart = new Date('2005-01-01 00:00:00');
        $oEnd = new Date('2005-01-01 02:00:00');

        Mock::generatePartial('ForecastZoneImpressions',
            $class = 'ForecastZoneImpressions' . rand(), array('_getDal'));

        Mock::generatePartial('MAX_Dal_Maintenance', $dalMock = 'MAX_Dal_Maintenance' . rand(),
            array('getZonesImpressionAverageByRange'));

        // create mock DAL
        $dal = &new $dalMock($this);
        /*
        [zone_id] => Array
               (
                   [operation_interval_id] => Array
                       (
                           ['average_impressions'] => Average Impressions
                       )
                   [operation_interval_id] => Array
                       (
                           ['average_impressions'] => Average Impressions
                       )
               )
        */
        $getZonesImpressionAverageByRange = array(
            1 => array(
                0 => array('average_impressions' => 100),
                1 => array('average_impressions' => 200),
                2 => array('average_impressions' => 300),
            ),
            2 => array(
                0 => array('average_impressions' => 200),
                1 => array('average_impressions' => 400),
                2 => array('average_impressions' => 600),
            )
        );
        $dal->setReturnValue('getZonesImpressionAverageByRange', $getZonesImpressionAverageByRange);

        $obj = & new $class($this);

        $obj->setReturnValue('_getDal', $dal);

        // run constructor to populate mock DAL
        $obj->ForecastZoneImpressions();

        // test method
        $obj->setZonesImpressionAverage($aZone, $oStart, $oEnd);

        // test values have been set in Zone objects
        foreach ($aZone as $key => $oZone) {
            $this->assertTrue($oZone->aOperationIntId[0]['averageImpressions'] == (($key + 1) * 100));
            $this->assertTrue($oZone->aOperationIntId[1]['averageImpressions'] == (($key + 1) * 200));
            $this->assertTrue($oZone->aOperationIntId[2]['averageImpressions'] == (($key + 1) * 300));
        }
    }

    function test_setZoneIntervalIdImpressionAverage()
    {
        Mock::generatePartial('Zone', $zoneMockClass = 'Zone' . rand(), array());
        $oZone = & new $zoneMockClass($this);

        $aResults = array(
            0 => array('average_impressions' => 500),
            1 => array('average_impressions' => 600),
            2 => array('average_impressions' => 650),
            3 => array('average_impressions' => 800),
            4 => array('average_impressions' => 400),
            23 => array('average_impressions' => 4658),
        );

        $task = &new ForecastZoneImpressions();
        $task->_setZoneIntervalIdImpressionAverage($oZone, $aResults);

        // test Zone object properties are populated
        foreach ($aResults as $interval => $val ) {
            $this->assertEqual($oZone->aOperationIntId[$interval]['averageImpressions'], $val['average_impressions']);
        }
    }

    function testGetTrendOperationIntervalStartOffset()
    {
        $task = &new ForecastZoneImpressions();
        $offsetStart = (ZONE_FORECAST_TREND_OFFSET + ZONE_FORECAST_TREND_OPERATION_INTERVALS - 1);
        $this->assertEqual($task->getTrendOperationIntervalStartOffset(), $offsetStart);
        $this->assertNotEqual($task->getTrendOperationIntervalStartOffset(), 0);
    }

}

?>
