<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.1                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: campaignhistory.plugin.php 746 2004-10-07 09:37:35Z andrew $
*/


/**
 * @package    MaxPlugins
 * @subpackage Reports
 * @author     Chris Nutting <chris@m3.net>
 * @author     Radek Maciaszek <radek@m3.net>
 */

require_once MAX_PATH . '/plugins/reports/Reports.php';

class Plugins_Reports_Agency_Report extends Plugins_Reports {
    
    function info() 
    {
    	global $strCampaignHistory, $strCampaign, $strPluginCampaign, $strDelimiter;
    	
    	include_once MAX_PATH . '/lib/max/Plugin/Translation.php';
        MAX_Plugin_Translation::init($this->module, $this->package);
           
    	//$startDate = date("Y/m/d", mktime(0,0,0,date("m")-1,1,date("Y")));
    	//$endDate = date("Y/m/d", mktime(0,0,0,date("m")-1, date
    	
    	$plugininfo = array (
    		"plugin-name"			=> MAX_Plugin_Translation::translate('Agency report', $this->module, $this->package),
    		"plugin-description"	=> MAX_Plugin_Translation::translate('This report generates an agency-wide report on' . 
    		                           'all activity between the specified dates (Date range is <b>inclusive</b>)', $this->module, $this->package),
    		'plugin-category'     => 'agency',
    		'plugin-category-name'	=> MAX_Plugin_Translation::translate('Agency Reports', $this->module, $this->package),
    		"plugin-author"			=> "Chris Nutting",
    		"plugin-export"			=> "csv",
    		"plugin-authorize"		=> phpAds_Admin+phpAds_Agency,
    		"plugin-import"			=> array (
    			"delimiter"		=> array (
    				"title"					=> $strDelimiter,
    				"type"					=> "edit",
    				"size"					=> 1,
    				"default"				=> "," 
    				),
    			"start_date"		=> array (
    				"title"					=> "Start Date (YYYY/MM/DD)",
    				"type"					=> "edit",
    				"size"					=> 10,
    				"default"				=> date("Y/m/d",mktime (0,0,0,date("m")-1,1,date("Y"))), 
    				),
    			"end_date"		=> array (
    				"title"					=> "End Date (YYYY/MM/DD)",
    				"type"					=> "edit",
    				"size"					=> 10,
    				"default"				=> date("Y/m/d", mktime(0,0,0,date("m")-1,date("t", date("m")-1),date("Y"))), 
    				)
    			)
    		);
    	
    	return ($plugininfo);
    }

    
    
    /*********************************************************/
    /* Private plugin function                               */
    /*********************************************************/
    
    function execute($delimiter=",", $start_date, $end_date)
    {
        $conf = & $GLOBALS['_MAX']['CONF'];
    	global $date_format;
    	global $strCampaign, $strTotal, $strDay, $strImpressions, $strClicks, $strCTRShort, $strConversions, $strCNVRShort;
    	
    	// Format the start and end dates
        $dbStart = date("Y-m-d", strtotime($start_date));
        $dbEnd   = date("Y-m-d", strtotime($end_date));
        $start = date("Y/m/d", strtotime($start_date));
        $end   = date("Y/m/d", strtotime($end_date));
    	
    	
    	$clientid = phpAds_getUserID();
    	
    	header ("Content-type: application/csv\nContent-Disposition: inline; filename=AgencyStats_".$dbStart."-".$dbEnd.".csv");
    	
    	$left_query = "
    	SELECT
        a.clientname AS advertisername,
            c.campaignname AS campaignname,
            c.campaignid AS campaignid,
            p.name AS publishername,
            z.zonename AS zonename,
            z.width AS width,
            z.height AS height,
            SUM(s.impressions) AS views,
            SUM(s.clicks) AS clicks,
            SUM(s.conversions) AS conversions
    
    	FROM
    	    {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']} AS s,
            {$conf['table']['prefix']}{$conf['table']['banners']} AS b,
            {$conf['table']['prefix']}{$conf['table']['campaigns']} AS c,
            {$conf['table']['prefix']}{$conf['table']['clients']} AS a
    
    	LEFT JOIN
            {$conf['table']['prefix']}{$conf['table']['zones']} AS z
          ON
            s.zone_id = z.zoneid
    
    	LEFT JOIN
            {$conf['table']['prefix']}{$conf['table']['affiliates']} AS p
          ON
            z.affiliateid = p.affiliateid
    
    	WHERE
            s.day >= '".$dbStart."'
          AND
            s.day <= '".$dbEnd."'
          AND
            b.bannerid = s.ad_id
          AND
            c.campaignid = b.campaignid
          AND
            a.clientid = c.clientid
          ";
        
    	$left_query .= (phpAds_isUser(phpAds_Agency)) ? " AND a.agencyid = '".$clientid."'" : '';
    
        $left_query .= "
    	GROUP BY
            b.campaignid,
            s.zone_id
    
    	ORDER BY
            advertisername,
            campaignname,
            publishername,
            zonename;
    	";
    	
    	$res_banners = phpAds_dbQuery($left_query) or phpAds_sqlDie();
    
    	echo "Advertiser".$delimiter."Campaign".$delimiter."Publisher".$delimiter."Zone".$delimiter."Size".$delimiter;
    	echo "Views".$delimiter."Clicks".$delimiter,"Conversions\n\n";
    	
    	$totals['views'] = 0;
    	$totals['clicks'] = 0;
    	$totals['conversions'] = 0;
    	
    	
    	while ($row_banners = phpAds_dbFetchArray($res_banners))
    	{
    	    $row_banners = str_replace($delimiter, ' ', $row_banners);
    	    
    	    ($row_banners['width'] == -1) ? $width = '*' : $width = $row_banners['width'];
    	    ($row_banners['height'] == -1) ? $height = '*' : $height = $row_banners['height'];
    	    
    	    if ($row_banners['width'] == '') $width = '?';
    	    if ($row_banners['height'] == '') $height = '?';
    	    
    	    
    	    if ($row_banners['publishername'] == '') $row_banners['publishername'] = 'Unknown';
    	    if ($row_banners['zonename'] == '') $row_banners['zonename'] = 'Unknown';
    	    
    	    
    	    echo $row_banners['advertisername'].$delimiter;
    	    echo $row_banners['campaignname'].$delimiter;
    	    echo $row_banners['publishername'].$delimiter;
    	    echo $row_banners['zonename'].$delimiter;
    	    echo $width."x".$height.$delimiter;
    	    echo $row_banners['views'].$delimiter;
    	    echo $row_banners['clicks'].$delimiter;
    	    echo $row_banners['conversions'].$delimiter;
    	    
    	    echo "\n";
            $totals['views'] += $row_banners['views'];
            $totals['clicks'] += $row_banners['clicks'];
            $totals['conversions'] += $row_banners['conversions'];
    	}
    	
    	echo "\nTotals".$delimiter.$delimiter.$delimiter.$delimiter.$delimiter.$totals['views'].$delimiter.$totals['clicks'].$delimiter.$totals['conversions']."\n";
    }
}

?>
