<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: affiliates.plugin.php 4787 2006-05-04 17:56:20Z matteo@beccati.com $
*/

/**
 * @package    MaxPlugins
 * @subpackage StatsFields
 * @author     Matteo Beccati <matteo@beccati.com>
 */

require_once MAX_PATH . '/plugins/statsFields/statsFields.php';
require_once MAX_PATH . '/lib/max/Plugin/Translation.php';

/**
 * Stats fields plugin
 *
 * @package    MaxPlugins
 * @subpackage Plugins_StatsFields
 * @author     Matteo Beccati <matteo@beccati.com>
 * @abstract
 */
class Plugins_StatsFields_affiliates_affiliates extends Plugins_StatsFields
{
    /**
     * Constructor
     */
    function Plugins_StatsFields_affiliates_affiliates() {
        // Set module and package because they aren't set when running the constructor method
        $this->module  = 'statsFields';
        $this->package = 'affiliates';
        
        $this->_fields = array(
            'sum_revenue' => array('name' => MAX_Plugin_Translation::translate('Rev.', $this->module, $this->package),
                                   'format' => 'default'),
            'sum_cost'    => array('name' => MAX_Plugin_Translation::translate('Cost', $this->module, $this->package),
                                   'format' => 'default'),
            'sum_bv'      => array('name' => MAX_Plugin_Translation::translate('BV', $this->module, $this->package),
                                   'format' => 'default'),
            'sum_revcpc'  => array('name' => MAX_Plugin_Translation::translate('Rev. CPC', $this->module, $this->package),
                                   'format' => 'currency'),
            'sum_costcpc' => array('name' => MAX_Plugin_Translation::translate('Cost CPC', $this->module, $this->package),
                                   'format' => 'currency')
        );
        $this->_prefs = array(
            'sum_revenue' => array('name' => MAX_Plugin_Translation::translate('Show Revenue column', $this->module, $this->package),
                                   'var' => 'gui_show_revenue'),
            'sum_cost'    => array('name' => MAX_Plugin_Translation::translate('Show Cost column', $this->module, $this->package),
                                   'var' => 'gui_show_cost'),
            'sum_bv'      => array('name' => MAX_Plugin_Translation::translate('Show BV column', $this->module, $this->package),
                                   'var' => 'gui_show_bv'),
            'sum_revcpc'  => array('name' => MAX_Plugin_Translation::translate('Show Rev. CPC column', $this->module, $this->package),
                                   'var' => 'gui_show_revcpc'),
            'sum_costcpc' => array('name' => MAX_Plugin_Translation::translate('Show Cost CPC column', $this->module, $this->package),
                                   'var' => 'gui_show_costcpc')
        );
    }

    /**
     * Return name of plugin
     *
     * @abstract
     * @return string A string describing the class.
     */
    function getName()
    {
        return 'Affiliate columns';
    }

    
    function addQueryParams(&$aParams)
    {
        $aParams['add_columns']['SUM(s.total_revenue)']      = 'sum_revenue';
        $aParams['add_columns']['SUM(s.total_cost)']         = 'sum_cost';
        $aParams['add_columns']['SUM(s.total_basket_value)'] = 'sum_bv';
    }

    /**
     * Generate CTR and SR ratios
     *
     * @static
     *
     * @param array Row of stats
     */
    function summarizeStats(&$row)
    {
        $row['sum_revcpc']   = $row['sum_clicks'] ? $row['sum_revenue'] / $row['sum_clicks'] : 0;
        $row['sum_costcpc']  = $row['sum_clicks'] ? $row['sum_cost'] / $row['sum_clicks'] : 0;
    }
}

?>
